Attribute VB_Name = "DataAccess"
Option Private Module
Option Explicit

Public Function ExtractGraphNodes(target As Visio.Page) As Collection
  Dim iNodes As New Collection
  Dim aShape As Visio.Shape
  For Each aShape In target.Shapes
    If aShape.OneD Or aShape.MasterShape Is Nothing Then _
      GoTo NEXT_SHAPE
    Dim sMaster$: sMaster = aShape.MasterShape.Name
    If sMaster = SHAPE_OPERATION _
        Or sMaster = SHAPE_SCHEMA _
        Or sMaster = SHAPE_PROXY Then _
      Call iNodes.Add(aShape)
NEXT_SHAPE:
  Next aShape
  Set ExtractGraphNodes = iNodes
End Function

Public Function ExtractSchemas(target As Visio.Page) As Collection
  Dim iSchemas As New Collection
  Dim aShape As Visio.Shape
  For Each aShape In target.Shapes
    If aShape.OneD Or aShape.MasterShape Is Nothing Then _
      GoTo NEXT_SHAPE
    Dim sMaster$: sMaster = aShape.MasterShape.Name
    If sMaster = SHAPE_SCHEMA Then _
      Call iSchemas.Add(aShape)
NEXT_SHAPE:
  Next aShape
  Set ExtractSchemas = iSchemas
End Function

Public Function GetTemplate(sFile$, sLocalFolder$) As String
  Dim fso As New Scripting.FileSystemObject
  
  GetTemplate = SERVER_TEMPLATES & "\" & sFile
  If fso.FileExists(GetTemplate) Then _
    Exit Function
  
  GetTemplate = sLocalFolder & "\" & sFile
  If fso.FileExists(GetTemplate) Then _
    Exit Function

  GetTemplate = TEMPLATE_PREFIX & "\" & sFile
End Function

Public Function TrimEnumerator(sText$) As String
  If IsNumerated(sText) Then
    Dim pos&: pos = VBA.InStr(1, sText, " ")
    TrimEnumerator = VBA.Right(sText, VBA.Len(sText) - pos)
  Else
    TrimEnumerator = sText
  End If
End Function

Public Function ExtractEnumerator(sText$) As Long
  If Not IsNumerated(sText) Then _
    Exit Function
  If sText Like "[?][?][?] *" Then _
    Exit Function
  Dim nPos&: nPos = VBA.InStr(1, sText, " ")
  If VBA.Left(sText, 1) = "!" Then
    ExtractEnumerator = CLng(VBA.Mid(sText, 4, nPos - 4))
  Else
    ExtractEnumerator = CLng(VBA.Mid(sText, 3, nPos - 3))
  End If
End Function

Public Function AppendEnumerator(sText$, nIndex&) As String
  AppendEnumerator = "" + Format(nIndex, "00") + " " + sText
End Function

Public Function IsElementary(shapeType As TElement) As Boolean
  Select Case (shapeType)
    Case T_EL_UNKNOWN: IsElementary = False
    Case T_EL_SCHEMA: IsElementary = False
    Case T_EL_PROXY: IsElementary = False
    Case T_EL_BLOCK: IsElementary = False
    Case T_EL_OPERATION: IsElementary = False
    Case T_EL_HEADER: IsElementary = False
    Case T_EL_MINIATURE: IsElementary = False
    Case Else: IsElementary = True
  End Select
End Function

Public Function GetShapeType(target As Visio.Shape) As TElement
  If target.MasterShape Is Nothing Then
    GetShapeType = T_EL_UNKNOWN
    Exit Function
  End If
  
  Select Case (target.MasterShape.Name)
    Case SHAPE_SCHEMA: GetShapeType = T_EL_SCHEMA
    Case SHAPE_PROXY: GetShapeType = T_EL_PROXY
    Case SHAPE_BLOCK: GetShapeType = T_EL_BLOCK
    Case SHAPE_OPERATION: GetShapeType = T_EL_OPERATION
    Case SHAPE_IDENTIFICATION: GetShapeType = T_EL_IDENTITY
    Case SHAPE_ELEMENT: GetShapeType = GetElementaryType(target)
    Case SHAPE_HEADER: GetShapeType = T_EL_HEADER
    Case SHAPE_MINIATURE: GetShapeType = T_EL_MINIATURE
    Case Else: GetShapeType = T_EL_UNKNOWN
  End Select
End Function

Public Function GetElementaryType(target As Visio.Shape) As TElement
  If Not target.CellExistsU(CELLSU_ELEMENT_TYPE, visExistsAnywhere) Then
    GetElementaryType = T_EL_COMMENT
    Exit Function
  End If
  
  Select Case (target.CellsU(CELLSU_ELEMENT_TYPE))
    Case T_SE_BASIC: GetElementaryType = T_EL_BASIC
    Case T_SE_DERIVED: GetElementaryType = T_EL_DERIVED
    Case T_SE_ASSERTION: GetElementaryType = T_EL_ASSERTION
    Case T_SE_COMMENT: GetElementaryType = T_EL_COMMENT
    Case T_SE_INTERPRETATION: GetElementaryType = T_EL_INTERPRETATION
    Case Else: GetElementaryType = T_EL_BASIC
  End Select
End Function

Public Function ExtractPowerLevels(target As Visio.Page) As Scripting.Dictionary
  Dim iGraph As CDS_Graph: Set iGraph = ScanGraph(ExtractGraphNodes(target), False)
  Dim iOrder As Collection: Set iOrder = GeotopSort(target, iGraph)
  Dim iReps As Scripting.Dictionary: Set iReps = ExtractRepresentatives(target)
  Dim estimator As New PowerEstimator
  Set ExtractPowerLevels = estimator.Estimate(iGraph, iOrder, iReps)
End Function

Public Function ExtractRepresentatives(target As Visio.Page) As Scripting.Dictionary
  Dim iReps As New Scripting.Dictionary
  
  Dim schemas As Collection: Set schemas = ExtractGraphNodes(target)
  Dim textReps As New Scripting.Dictionary
  Dim aShape As Visio.Shape
  Dim sText$
  For Each aShape In schemas
    If aShape.MasterShape.Name = SHAPE_SCHEMA Then
      sText = TrimEnumerator(aShape.Text)
      If Not textReps.Exists(sText) Then _
        Call textReps.Add(sText, aShape.ID)
    End If
  Next aShape
  
  For Each aShape In schemas
    If aShape.MasterShape.Name = SHAPE_PROXY Then
      sText = TrimEnumerator(aShape.Text)
      If textReps.Exists(sText) Then _
        Call iReps.Add(aShape.ID, textReps(sText))
    End If
  Next aShape
  
  Set ExtractRepresentatives = iReps
End Function

' =========
Private Function IsNumerated(sText$) As Boolean
  IsNumerated = True
  
  If sText Like "#### *" Or sText Like "!#### *" Then _
    Exit Function
  If sText Like "### *" Or sText Like "!### *" Then _
    Exit Function
  If sText Like "## *" Or sText Like "!## *" Then _
    Exit Function
  If sText Like "# *" Or sText Like "!# *" Then _
    Exit Function
  If sText Like "[?][?][?] *" Then _
    Exit Function
  
  IsNumerated = False
End Function
