Attribute VB_Name = "Main"
Option Explicit

Public Const PRODUCT_VERSION = "1.3.0"
Public Const PRODUCT_NAME = "Concept-Blocks"

Public g_VersionTimer As Long

Public Sub StartVersionCheck()
  g_VersionTimer = SetTimer(0, 0, CP_VERSION_MSG_DELAY, AddressOf OnVersionCheck)
End Sub

Public Function OnVersionCheck(ByVal nHwnd As Long, ByVal uMsg As Long, ByVal nEvent As Long, ByVal nTime As Long)
  Call KillTimer(0, g_VersionTimer)
  Call VersionValidate(PRODUCT_NAME, PRODUCT_VERSION)
End Function

Public Sub RunToggleHeaders()
Attribute RunToggleHeaders.VB_ProcData.VB_Invoke_Func = "t"
  Call GlobalUndo.BeginScope("")
  ThisDocument.DiagramServicesEnabled = visServiceStructureFull
  
  Dim iSelected As Visio.Selection: Set iSelected = ActiveWindow.Selection
  If iSelected.Count <> 0 Then
    Call ToggleHeadersSelected(iSelected)
  Else
    Call ToggleHeadersAll(ActivePage)
  End If
  
  ActiveWindow.Selection = iSelected
  
  ThisDocument.DiagramServicesEnabled = 0
  Call GlobalUndo.EndScope
End Sub

Public Sub RunWidthAdd10()
Attribute RunWidthAdd10.VB_ProcData.VB_Invoke_Func = "e"
  Call CC_WidthAdd10
End Sub

Public Sub RunWidthSub10()
Attribute RunWidthSub10.VB_ProcData.VB_Invoke_Func = "q"
  Call CC_WidthSub10
End Sub

Public Sub RunGotoLinkBegin()
Attribute RunGotoLinkBegin.VB_ProcData.VB_Invoke_Func = "Q"
  Call CC_GotoLinkBegin
End Sub

Public Sub RunGotoLinkEnd()
Attribute RunGotoLinkEnd.VB_ProcData.VB_Invoke_Func = "E"
  Call CC_GotoLinkEnd
End Sub

Public Sub RunToggleMiniatures()
Attribute RunToggleMiniatures.VB_ProcData.VB_Invoke_Func = "T"
  Call GlobalUndo.BeginScope(" ")
  
  Call ToggleMiniatures(ActivePage)
  Call ActiveWindow.DeselectAll

  Call GlobalUndo.EndScope
End Sub

Public Sub RunToggleHighlight()
Attribute RunToggleHighlight.VB_ProcData.VB_Invoke_Func = "R"
  Call GlobalUndo.BeginScope(" ")
  
  Call ToggleHighlight(ActiveWindow.Selection)

  Call GlobalUndo.EndScope
End Sub

Public Sub RunToggleInherited()
  Call GlobalUndo.BeginScope(" ")
  
  Call ToggleInherited(ActiveWindow.Selection)

  Call GlobalUndo.EndScope
End Sub

Public Sub RunCycleElementType()
Attribute RunCycleElementType.VB_ProcData.VB_Invoke_Func = "r"
  If ActiveWindow.Selection.Count = 0 Then
    Call UserInteraction.ShowMessage(EM_SELECT_ELEMENT)
    Exit Sub
  End If
  
  Call GlobalUndo.BeginScope("  ")
  
  Dim aShape As Visio.Shape
  For Each aShape In ActiveWindow.Selection
    Call CycleElementType(aShape)
  Next aShape
  
  Call GlobalUndo.EndScope
End Sub

Public Sub RunReflectParents()
  If ActiveWindow.Selection.Count <> 1 Then
    Call UserInteraction.ShowMessage(EM_SELECT_SCHEMA)
    Exit Sub
  End If
  
  Call GlobalUndo.BeginScope("")
  ThisDocument.DiagramServicesEnabled = visServiceStructureBasic

  Call ReflectParents(ActiveWindow.Selection(1))
  
  ThisDocument.DiagramServicesEnabled = 0
  Call GlobalUndo.EndScope
End Sub

Public Sub RunEnumerateSchemas()
  Call GlobalUndo.BeginScope(" ")
  Call EnumerateSchemasOn(ActivePage)
  Call GlobalUndo.EndScope
End Sub

Public Sub RunFixEnumeration()
  Call GlobalUndo.BeginScope(" ")
  Call FixEnumerationOn(ActivePage)
  Call GlobalUndo.EndScope
End Sub

Public Sub RunExportWord()
  Dim iExporter As New WordExporter
  If Not iExporter.Init(ThisDocument.Path) Then _
    Exit Sub
  
  Dim iPage As Visio.Page: Set iPage = ActivePage
  Dim iOrder As Collection: Set iOrder = GeotopSort(iPage, ScanGraph(ExtractGraphNodes(iPage), False))
  
  Call CSE_ProgressBar.Init(" ", maxVal:=iOrder.Count)
  Call CSE_ProgressBar.ShowModeless

  Call iExporter.Export(iPage, iOrder)
  
  Call Unload(CSE_ProgressBar)
  
  Call UserInteraction.ShowMessage(IM_EXPORT_OK)
End Sub

Public Sub RunExportExcel()
  Dim iExporter As New XLExporter
  If Not iExporter.Init(ThisDocument.Path) Then _
    Exit Sub
  
  Call CSE_ProgressBar.Init(" ", maxVal:=ActivePage.Shapes.Count)
  Call CSE_ProgressBar.ShowModeless
  Call iExporter.Export(ActivePage)
  Call Unload(CSE_ProgressBar)
  
  Call UserInteraction.ShowMessage(IM_EXPORT_OK)
End Sub

Public Sub RunToggleColorTheme()
  Call GlobalUndo.BeginScope("  ")
  Call ToggleThemeColors(ActivePage)
  Call GlobalUndo.EndScope
End Sub

Public Sub RunHelp()
  Dim wordWrap As New API_WordWrapper
  Dim helpDoc As Word.Document: Set helpDoc = wordWrap.OpenDocument(HELP_FILE_PATH, bReadOnly:=True)
  If helpDoc Is Nothing Then _
    Exit Sub
  With helpDoc
    .ActiveWindow.View.ReadingLayout = False
    Call .Application.Activate
  End With
End Sub

' ===== Context menu API ========
Public Function ToggleHeaderFor(target As Visio.Shape)
  Call GlobalUndo.BeginScope("")
  ThisDocument.DiagramServicesEnabled = visServiceStructureFull
  
  Dim iSelected As Visio.Selection: Set iSelected = ActiveWindow.Selection
  Dim bShow As Boolean: bShow = CellsGetValue(target, CELLSU_BLOCK_HEADER)
  Call ShowHeader(target, Not bShow)
  ActiveWindow.Selection = iSelected
  
  ThisDocument.DiagramServicesEnabled = 0
  Call GlobalUndo.EndScope
End Function

Public Function MinimizeListContainer(target As Visio.Shape)
  If target.Shapes.Count <> 2 Then
    Call UserInteraction.ShowMessage(EM_MINIMIZE_CONTAINERS_ONLY)
    Exit Function
  End If
  
  Call GlobalUndo.BeginScope("")
  
  Call target.Resize(visResizeDirS, target.Shapes(2).CellsU("Height") - target.CellsU("Height"), visInches)

  Call GlobalUndo.EndScope
End Function

Public Function FixElementsWidthFor(target As Visio.Shape)
  ThisDocument.DiagramServicesEnabled = visServiceStructureBasic
  
  Dim itemWidth As Double: itemWidth = target.CellsU("Width") - 2 * target.CellsU(CELLSU_CONTAINER_MARGINS)
  Dim memberID As Variant
  Dim itemShape As Visio.Shape
  For Each memberID In target.ContainerProperties.GetListMembers
    Set itemShape = target.ContainingPage.Shapes.ItemFromID(memberID)
    Call itemShape.Resize(visResizeDirE, itemWidth - itemShape.CellsU("Width"), visInches)
  Next memberID
  
  ThisDocument.DiagramServicesEnabled = 0
End Function

Public Function FitToContentsFor(target As Visio.Shape)
  Dim tmp&: tmp = target.ContainerProperties.ResizeAsNeeded
  target.ContainerProperties.ResizeAsNeeded = visContainerAutoResizeExpandContract
  DoEvents
  target.ContainerProperties.ResizeAsNeeded = tmp
End Function

Public Function CreateProxyFor(target As Visio.Shape)
  Call GlobalUndo.BeginScope(" ")
  ThisDocument.DiagramServicesEnabled = visServiceStructureFull

  Dim iProxy As Visio.Shape: Set iProxy = CreateProxyShape(target)
  Call ActiveWindow.Select(iProxy, visSelect)
  
  ThisDocument.DiagramServicesEnabled = 0
  Call GlobalUndo.EndScope
End Function

Public Function CreateOperationFor(target As Visio.Shape)
  If CountInboundConnects(target) < 1 Then
    Call UserInteraction.ShowMessage(EM_OPERANDS_MISSING)
    Exit Function
  End If
  
  Call GlobalUndo.BeginScope(" ")
  ThisDocument.DiagramServicesEnabled = visServiceStructureFull
  
  Dim iOperation As Visio.Shape: Set iOperation = CreateOperationShape(target)
  Call ActiveWindow.Select(iOperation, visSelect)
  
  ThisDocument.DiagramServicesEnabled = 0
  Call GlobalUndo.EndScope
End Function

Public Function RenameSchema(target As Visio.Shape)
  Dim sInput$: sInput = UserInteraction.PromptInput("   ", sInitial:=TrimEnumerator(target.Text))
  If sInput = vbNullString Then _
    Exit Function
    
  Call GlobalUndo.BeginScope(" ")
  Call RenameSchemaDeep(target, sInput)
  Call GlobalUndo.EndScope
End Function

' ==========
Private Function CountInboundConnects(target As Visio.Shape) As Long
  Dim nCount&
  Dim iConnect As Visio.Connect
  For Each iConnect In target.FromConnects
    If target = iConnect.FromSheet.Connects(2).ToSheet Then _
      nCount = nCount + 1
  Next iConnect
  CountInboundConnects = nCount
End Function
