Attribute VB_Name = "MainImpl"
Option Private Module
Option Explicit

Public Function ClearAll()
  Call VsoClearPage(ActivePage)
End Function

Public Function CycleElementType(target As Visio.Shape)
  If Not target.CellExistsU(CELLSU_ELEMENT_TYPE, visExistsAnywhere) Then _
    Exit Function
    
  Select Case (target.CellsU(CELLSU_ELEMENT_TYPE))
    Case T_SE_BASIC: target.CellsU(CELLSU_ELEMENT_TYPE) = 2
    Case T_SE_DERIVED: target.CellsU(CELLSU_ELEMENT_TYPE) = 4
    Case T_SE_ASSERTION: target.CellsU(CELLSU_ELEMENT_TYPE) = 0
    Case T_SE_COMMENT: target.CellsU(CELLSU_ELEMENT_TYPE) = 3
    Case T_SE_INTERPRETATION: target.CellsU(CELLSU_ELEMENT_TYPE) = 1
    Case Else: target.CellsU(CELLSU_ELEMENT_TYPE) = 1
  End Select
End Function

Public Function ToggleInherited(targets As Visio.Selection)
  Dim aShape As Visio.Shape
  For Each aShape In targets
    If aShape.CellExistsU(CELLSU_IS_INHERITED, visExistsAnywhere) Then _
      aShape.CellsU(CELLSU_IS_INHERITED).FormulaU = IIf(aShape.CellsU(CELLSU_IS_INHERITED).Result(visLogical), "FALSE", "TRUE")
  Next aShape
End Function

Public Function ToggleHighlight(targets As Visio.Selection)
  Dim aShape As Visio.Shape
  For Each aShape In targets
    If aShape.CellExistsU(CELLSU_IS_HIGHTLIGHT, visExistsAnywhere) Then _
      aShape.CellsU(CELLSU_IS_HIGHTLIGHT).FormulaU = IIf(aShape.CellsU(CELLSU_IS_HIGHTLIGHT).Result(visLogical), "FALSE", "TRUE")
  Next aShape
End Function

Public Function ToggleMiniatures(target As Visio.Page)
  If ClearMiniatures(target) = 0 Then _
    Call CreateMiniatures(target)
End Function

Public Function EnumerateSchemasOn(target As Visio.Page)
  Dim shapeOrder As Collection: Set shapeOrder = GeotopSort(target, ScanGraph(ExtractGraphNodes(target), False))
  Dim indexBase As Scripting.Dictionary: Set indexBase = EnumerateSchemas(shapeOrder)
  Call EnumerateProxies(shapeOrder, indexBase)
End Function

Public Function FixEnumerationOn(target As Visio.Page)
  Dim shapeOrder As Collection: Set shapeOrder = GeotopSort(target, ScanGraph(ExtractGraphNodes(target), False))
  Dim indexBase As New Scripting.Dictionary: Set indexBase = FixSchemas(target, shapeOrder)
  Call EnumerateProxies(shapeOrder, indexBase)
End Function

Public Function ReflectParents(target As Visio.Shape)
  Dim iGraph As CDS_Graph: Set iGraph = ScanGraph(CColl(target), True)
  
  Dim dBaseX As Double: dBaseX = target.CellsU("PinX")
  
  Dim nShapeID As Variant
  Dim aShape As Visio.Shape
  For Each nShapeID In iGraph.Nodes
    Set aShape = target.Parent.Shapes.ItemFromID(nShapeID)
    If aShape.MasterShape.Name = SHAPE_OPERATION Then _
      Call SwapIdentifications(aShape)
    aShape.CellsU("PinX") = dBaseX + (dBaseX - aShape.CellsU("PinX"))
  Next nShapeID
End Function

Public Function ToggleThemeColors(iPage As Visio.Page)
  Dim aShape As Visio.Shape
  Dim bFirst As Boolean: bFirst = True
  Dim bMakeLighter As Boolean
  For Each aShape In iPage.Shapes
    If aShape.MasterShape.Name <> SHAPE_ELEMENT Then _
      GoTo NEXT_SHAPE
    
    If bFirst Then
      bFirst = False
      bMakeLighter = aShape.CellsU("User.Color1").FormulaU <> "RGB(233,244,226)"
    End If
    
    Call ToggleElementColor(aShape, bMakeLighter)
    
NEXT_SHAPE:
  Next aShape
End Function

Public Function CreateOperationShape(target As Visio.Shape) As Visio.Shape
  Dim iOperation As Visio.Shape
  With target
    Set iOperation = .ContainingPage.Drop(FindMaster(.Document, MASTER_OPERATION), .CellsU("PinX"), .CellsU("PinY"))
    iOperation.CellsU("PinY") = .CellsU("PinY") + .CellsU("Height") / 2# + iOperation.CellsU("Height") / 2# + OPERATION_GAP
  End With
  
  Dim connection As Visio.Connect
  For Each connection In target.FromConnects
    If target = connection.FromSheet.Connects(2).ToSheet Then _
      Call connection.FromSheet.CellsU("EndX").GlueTo(iOperation.CellsSRC(7, 1, 0))
  Next connection
  
  Call iOperation.AutoConnect(target, visAutoConnectDirNone)
  Set CreateOperationShape = iOperation
End Function

Public Function CreateProxyShape(target As Visio.Shape) As Visio.Shape
  Dim newShape As Visio.Shape
  With target
    Set newShape = .ContainingPage.Drop(FindMaster(.Document, MASTER_PROXY), .CellsU("PinX"), .CellsU("PinY"))
    newShape.Text = .Text
    newShape.CellsU("PinX") = .CellsU("PinX") + .CellsU("Width") / 2# + newShape.CellsU("Width") / 2#
  End With
  Set CreateProxyShape = newShape
End Function

Public Function ToggleHeadersSelected(iSelected As Visio.Selection)
  Dim aShape As Visio.Shape
  Dim bShow As Boolean
  For Each aShape In iSelected
    If GetShapeType(aShape) <> T_EL_BLOCK Then _
      GoTo NEXT_SHAPE

    bShow = CellsGetValue(aShape, CELLSU_BLOCK_HEADER)
    Call ShowHeader(aShape, Not bShow)
    
NEXT_SHAPE:
  Next aShape
End Function

Public Function ToggleHeadersAll(iPage As Visio.Page)
  Dim aShape As Visio.Shape
  Dim bFirst As Boolean: bFirst = True
  Dim bShow As Boolean
  For Each aShape In iPage.Shapes
    If GetShapeType(aShape) <> T_EL_BLOCK Then _
      GoTo NEXT_SHAPE
    
    If bFirst Then
      bFirst = False
      bShow = CellsGetValue(aShape, CELLSU_BLOCK_HEADER)
      bShow = Not bShow
    End If
    
    Call ShowHeader(aShape, bShow)
    
NEXT_SHAPE:
  Next aShape
End Function

Public Function ShowHeader(target As Visio.Shape, bShow As Boolean) As Visio.Shape
  Call ResetHeader(target)
  Call CellsSetValue(target, CELLSU_BLOCK_HEADER, bShow)
  If bShow Then _
    Set ShowHeader = CreateHeader(target)
End Function

Public Function GeotopSort(iPage As Visio.Page, iGraph As CDS_Graph) As Collection
  If iGraph.Size = 0 Then
    Set GeotopSort = New Collection
    Exit Function
  End If
  
  Dim ids() As Long: ids = ToLongArray(iGraph.Nodes)
  Call VsoGeometricSort(ids, iPage)
  
  Dim iSorter As New API_GraphOrdering: Call iSorter.Init(iGraph)
  Set GeotopSort = iSorter.SortLayers(FromArray(ids))
End Function

Public Function RenameSchemaDeep(target As Visio.Shape, sInput$)
  Dim sPrefix$: sPrefix = target.Text
  Dim sName$: sName = TrimEnumerator(target.Text)
  If sName = sPrefix Then
    sPrefix = vbNullString
  Else
    sPrefix = VBA.Left(sPrefix, VBA.Len(sPrefix) - VBA.Len(sName))
  End If
  
  target.Text = sPrefix & sInput
  Call RenameProxies(target.Parent, sPrefix, sName, sInput)
End Function

' ===========
Private Function ResetHeader(target As Visio.Shape)
  Dim memberID As Variant
  Dim iPage As Visio.Page: Set iPage = target.ContainingPage
  Dim itemShape As Visio.Shape
  For Each memberID In target.ContainerProperties.GetMemberShapes(visContainerFlagsDefault)
    Set itemShape = iPage.Shapes.ItemFromID(memberID)
    If GetShapeType(itemShape) = T_EL_HEADER Then
      Call itemShape.Delete
      Exit Function
    End If
  Next memberID
End Function

Private Function CreateHeader(target) As Visio.Shape
  Dim newShape As Visio.Shape
  With target
    Set newShape = .ContainingPage.Drop(FindMaster(.Document, MASTER_HEADER), .CellsU("PinX"), .CellsU("PinY"))
    newShape.CellsU("LockTextEdit") = 0
    newShape.Text = .Text
    newShape.CellsU("LockTextEdit") = 1
    newShape.CellsU("Width") = IIf(HEADER_MAX_WIDTH > .CellsU("Width") - HEADER_MARGINS, .CellsU("Width") - HEADER_MARGINS, HEADER_MAX_WIDTH)
  End With
  Set CreateHeader = newShape
End Function

Private Function ClearMiniatures(target As Visio.Page) As Long
  Dim cDelete As New Collection
  Dim iShape As Visio.Shape
  For Each iShape In target.Shapes
    If GetShapeType(iShape) = T_EL_MINIATURE Then _
      Call cDelete.Add(iShape)
  Next iShape
  
  For Each iShape In cDelete
    Call iShape.Delete
  Next iShape
  
  ClearMiniatures = cDelete.Count
End Function

Private Function CreateMiniatures(target As Visio.Page)
  Dim sText$
  Dim dHeight As Double
  Dim iMiniature As Visio.Shape
  Dim iSchema As Visio.Shape
  For Each iSchema In ExtractSchemas(target)
    Set iMiniature = target.Drop(FindMaster(iSchema.Document, MASTER_MINIATURE), iSchema.CellsU("PinX"), iSchema.CellsU("PinY"))
    iMiniature.CellsU("Width") = iSchema.CellsU("Width") + 2 * MINIATURE_OFFSET
    iMiniature.Shapes(2).Text = iSchema.Text
    
    dHeight = 0
    sText = ""
    
    Dim nElement As Variant
    Dim iElement As Visio.Shape
    Dim elems As Variant: elems = iSchema.ContainerProperties.GetListMembers
    For Each nElement In elems
      Set iElement = target.Shapes.ItemFromID(nElement)
      If CellsGetValue(iElement, CELLSU_IS_HIGHTLIGHT) Then
        dHeight = dHeight + iElement.CellsU("Height")
        If sText = vbNullString Then
          sText = iElement.Text
        Else
          sText = sText & vbNewLine & iElement.Text
        End If
      End If
    Next nElement
    
    iMiniature.Shapes(1).Text = sText
    iMiniature.CellsU("Height") = iMiniature.Shapes(2).CellsU("Height") + dHeight * MINIATURE_HEIGHT_MULTIPLIER
    iMiniature.CellsU("PinY") = iSchema.CellsU("PinY") + iSchema.CellsU("Height") / 2# - iMiniature.CellsU("Height") / 2# + MINIATURE_OFFSET
    
  Next iSchema
End Function

Private Function SwapIdentifications(target As Visio.Shape)
  Dim memberID As Variant
  Dim member As Visio.Shape
  For Each memberID In target.ContainerProperties.GetListMembers
    Set member = target.ContainingPage.Shapes.ItemFromID(memberID)
    If member.MasterShape.Name = SHAPE_IDENTIFICATION Then
      Dim sText$: sText = member.Shapes(1).Text
      member.Shapes(1).Text = member.Shapes(2).Text
      member.Shapes(2).Text = sText
    End If
  Next memberID
End Function

Private Function EnumerateSchemas(shapeOrder As Collection) As Scripting.Dictionary
  Dim indexBase As New Scripting.Dictionary
  Dim nNext&: nNext = 1
  Dim shapeID As Variant
  Dim aShape As Visio.Shape
  For Each shapeID In shapeOrder
    Set aShape = ActivePage.Shapes.ItemFromID(shapeID)
    If aShape.MasterShape.Name = SHAPE_SCHEMA Then
      Dim sText$: sText = TrimEnumerator(aShape.Text)
      If Not indexBase.Exists(sText) Then
        Call indexBase.Add(sText, nNext)
        nNext = nNext + 1
        aShape.Text = AppendEnumerator(sText, indexBase(sText))
      Else
        aShape.Text = "!" + AppendEnumerator(sText, indexBase(sText))
      End If
    End If
  Next shapeID
  Set EnumerateSchemas = indexBase
End Function

Private Function EnumerateProxies(iOrder As Collection, indexBase As Scripting.Dictionary)
  Dim shapeID As Variant
  Dim aShape As Visio.Shape
  For Each shapeID In iOrder
    Set aShape = ActivePage.Shapes.ItemFromID(shapeID)
    If aShape.MasterShape.Name = SHAPE_PROXY Then
      Dim sText$: sText = TrimEnumerator(aShape.Text)
      If Not indexBase.Exists(sText) Then
        aShape.Text = "??? " + sText
      Else
        aShape.Text = AppendEnumerator(sText, indexBase(sText))
      End If
    End If
  Next shapeID
End Function

Private Function FixSchemas(iPage As Visio.Page, iOrder As Collection) As Scripting.Dictionary
  Dim indexBase As New Scripting.Dictionary
  Dim nMaxIndex&: nMaxIndex = 0
 
  Dim sText$
  Dim aShape As Visio.Shape
  Dim shapeID As Variant
  For Each shapeID In iOrder
    Set aShape = iPage.Shapes.ItemFromID(shapeID)
    If aShape.MasterShape.Name <> SHAPE_SCHEMA Then _
      GoTo NEXT_SHAPE
    
    Dim enumerator&: enumerator = ExtractEnumerator(aShape.Text)
    If enumerator = 0 Then _
      GoTo NEXT_SHAPE
    If enumerator > nMaxIndex Then _
      nMaxIndex = enumerator
    
    sText = TrimEnumerator(aShape.Text)
    If Not indexBase.Exists(sText) Then
      Call indexBase.Add(sText, enumerator)
    Else
      aShape.Text = "!" + AppendEnumerator(sText, enumerator)
    End If
NEXT_SHAPE:
  Next shapeID
  
  Dim nextIndex&: nextIndex = nMaxIndex + 1
  For Each shapeID In iOrder
    Set aShape = iPage.Shapes.ItemFromID(shapeID)
    If aShape.MasterShape.Name = SHAPE_SCHEMA Then
      sText = TrimEnumerator(aShape.Text)
      If Not indexBase.Exists(sText) Then
        Call indexBase.Add(sText, nextIndex)
        nextIndex = nextIndex + 1
        aShape.Text = AppendEnumerator(sText, indexBase(sText))
      End If
    End If
  Next shapeID
  
  Set FixSchemas = indexBase
End Function

Private Function RenameProxies(iPage As Visio.Page, sPrefix$, sOldName$, sNewName$)
  Dim aShape As Visio.Shape
  For Each aShape In iPage.Shapes
    If aShape.MasterShape.Name = SHAPE_PROXY Then _
      If TrimEnumerator(aShape.Text) = sOldName Then _
        aShape.Text = sPrefix & sNewName
  Next aShape
End Function

Private Function ToggleElementColor(target As Visio.Shape, bMakeLighter As Boolean)
  With target
    If bMakeLighter Then
      .CellsU("User.Color1").FormulaU = "RGB(233,244,226)"
      .CellsU("User.Color2").FormulaU = "RGB(213,241,255)"
      .CellsU("User.Color3").FormulaU = "RGB(238,227,240)"
      .CellsU("User.Color4").FormulaU = "RGB(255,216,213)"
    Else
      .CellsU("User.Color1").FormulaU = "="
      .CellsU("User.Color2").FormulaU = "="
      .CellsU("User.Color3").FormulaU = "="
      .CellsU("User.Color4").FormulaU = "="
    End If
  End With
End Function
