VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "XLExporter"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private app_ As API_XLWrapper
Private outLinks_ As Excel.Worksheet
Private outShapes_ As Excel.Worksheet

Private powers_ As Scripting.Dictionary
Private nLink_ As Long
Private nShape_ As Long

Public Function Init(sLocalTemplates$) As Boolean
  Set app_ = New API_XLWrapper
  Dim sTemplate$: sTemplate = GetTemplate(XL_TEMPLATE_NAME, sLocalTemplates)
  Init = Not app_.NewDocument(sTemplate, bDefaultIfFail:=True) Is Nothing
  If Not Init Then _
    Exit Function
  If sTemplate = vbNullString Then _
    Call PrepareDefaultOutput
  Set outLinks_ = app_.Document.Sheets(XL_SHEET_LINKS)
  Set outShapes_ = app_.Document.Sheets(XL_SHEET_SHAPES)
End Function

Public Function Export(iSource As Visio.Page)
  Call app_.PauseUI
  
  Set powers_ = ExtractPowerLevels(iSource)
  nLink_ = 2
  nShape_ = 2
  Dim aShape As Visio.Shape
  For Each aShape In iSource.Shapes
    If aShape.OneD Then
      If aShape.Connects.Count = 2 Then
        Call ExportLink(aShape)
      End If
    Else
      Call ExportShape(aShape)
    End If
    Call CSE_ProgressBar.IncrementA
  Next aShape
  
  Call app_.ResumeUI
End Function

' ==========
Private Function PrepareDefaultOutput()
  Call UserInteraction.ShowMessage(EM_MISSING_TEMPLATE)
  With app_.Document
    .Sheets.Add
    .Sheets(1).Name = XL_SHEET_SHAPES
    .Sheets(2).Name = XL_SHEET_LINKS
  End With
End Function

Private Function ExportLink(target As Visio.Shape)
  outLinks_.Cells(nLink_, S_XLL_ID) = target.ID
  outLinks_.Cells(nLink_, S_XLL_SOURCE) = target.Connects(1).ToSheet.ID
  outLinks_.Cells(nLink_, S_XLL_DESTINATION) = target.Connects(2).ToSheet.ID
  outLinks_.Cells(nLink_, S_XLL_TEXT) = target.Text
  nLink_ = nLink_ + 1
End Function

Private Function ExportShape(target As Visio.Shape) As Boolean
  Dim nType As TElement: nType = GetShapeType(target)
  If nType = T_EL_HEADER Or nType = T_EL_MINIATURE Then _
    Exit Function
  
  Dim bInherited As Boolean: bInherited = CellsGetValue(target, CELLSU_IS_INHERITED)
  Dim bHighlighted As Boolean: bHighlighted = CellsGetValue(target, CELLSU_IS_HIGHTLIGHT)
  Dim vParent As Visio.Shape: Set vParent = GetContainingShape(target)
  
  outShapes_.Cells(nShape_, S_XLS_ID) = target.ID
  outShapes_.Cells(nShape_, S_XLS_TYPE) = nType
  If powers_.Exists(target.ID) And nType = T_EL_SCHEMA Then _
    outShapes_.Cells(nShape_, S_XLS_POWER) = powers_(target.ID)
  
  If IsElementary(nType) Then
    outShapes_.Cells(nShape_, S_XLS_INHERITED) = IIf(bInherited, "", "")
    outShapes_.Cells(nShape_, S_XLS_HIGHLIGHT) = IIf(bHighlighted, "", "")
  End If
  
  outShapes_.Cells(nShape_, S_XLS_PINX) = target.CellsU("PinX").Result(visMillimeters)
  outShapes_.Cells(nShape_, S_XLS_PINY) = target.CellsU("PinY").Result(visMillimeters)
  outShapes_.Cells(nShape_, S_XLS_WIDTH) = target.CellsU("Width").Result(visMillimeters)
  outShapes_.Cells(nShape_, S_XLS_HEIGHT) = target.CellsU("Height").Result(visMillimeters)
  
  If Not vParent Is Nothing Then _
    outShapes_.Cells(nShape_, S_XLS_CONTAINED) = vParent.ID
  
  If nType <> T_EL_IDENTITY Then
    outShapes_.Cells(nShape_, S_XLS_TEXT1) = target.Text
  Else
    outShapes_.Cells(nShape_, S_XLS_TEXT1) = target.Shapes(1).Text
    outShapes_.Cells(nShape_, S_XLS_TEXT2) = target.Shapes(2).Text
  End If
  
  nShape_ = nShape_ + 1
End Function
